/** 
         *   ElemGrafico: elemento grafico (sprite)
         *  
         *       @see Hardware Juego Imagen
         *       @author  Nacho
         */

/* --------------------------------------------------         
   Parte de Saboteur - Remake
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.01  14-May-2009  Nacho Cabanes
                      Version inicial, basada en la 0.08 de ElectroFreddy

 ---------------------------------------------------- */


public class ElemGrafico
{
    // ----- Atributos -----
    protected short x;
    protected short y;
    protected short incrX;
    protected short incrY;
    protected short alto;
    protected short ancho;
    protected bool activo;
    
    protected short xOriginal;  // Para llevar a su posicion inicial
    protected short yOriginal;

    // La imagen que se mostrar en pantalla, si es estatica
    protected Imagen  miImagen;
    
    // La secuencia de imagenes, si es animada
    protected Imagen[][] secuencia;
    protected byte fotogramaActual;
    protected byte direccion;
    public const byte ABAJO = 0;
    public const byte ARRIBA = 1;
    public const byte DERECHA = 2;
    public const byte IZQUIERDA = 3;
    
    bool contieneImagen;        // Si no contiene imagen, no se podra dibujar
    bool contieneSecuencia;     // La alternativa: imagenes multiples
        
    protected Juego miJuego;    // Para comunicar con el resto de elementos

    // ----- Operaciones -----

    /// Constructor vacio
    public  ElemGrafico()
    {
      contieneImagen = false;
      contieneSecuencia = false;
      activo = true;
      direccion = ABAJO;
      fotogramaActual = 0;
      secuencia = new Imagen[4][];
    }
    
    /// Constructor: Carga la imagen que representara a este elemento grafico
    public  ElemGrafico(string nombre)
    {
      CargarImagen(nombre);
      direccion = ABAJO;
      activo = true;
      fotogramaActual = 0;
      // Valores por defecto para ancho y alto
      ancho = 32;
      alto = 32;
    }
    
    /// Mueve el elemento grafico a otra posicion
    public  void MoverA(short nuevaX, short nuevaY)
    {
      x = nuevaX;
      y = nuevaY;
      if ( (xOriginal==0) && (yOriginal==0) )
      {
        xOriginal = nuevaX;
        yOriginal = nuevaY;
      }
    }
    
    /// Cambia la velocidad (incrX e incrY) de un elemento
    public  void SetVelocidad(short vX, short vY)
    {
      incrX = vX;
      incrY = vY;
    }


    /// Carga la imagen que representara a este elemento grafico
    public  void CargarImagen(string nombre)
    {
      miImagen = new Imagen();
      miImagen.Cargar(nombre);
      contieneImagen = true;
      contieneSecuencia = false;
    }

    
    /// Carga una secuencia de imagenes para un elemento animado
    public void CargarSecuencia(byte direcc, string[] nombres)
    {
      contieneImagen = true;
      contieneSecuencia = true;
      byte tamanyo = (byte) nombres.Length;
      secuencia[direcc] = new Imagen[tamanyo];
      for (byte i=0; i< nombres.Length; i++) {
        secuencia[direcc][i] = new Imagen(nombres[i]);
        }
      // Valores por defecto para ancho y alto
      ancho = 32;
      alto = 32;
    }

    /// Mueve el elemento grafico a otra posicion
    public  void CambiarDireccion(byte nuevaDir)
    {
      if (direccion != nuevaDir)
      {
        direccion = nuevaDir;
        fotogramaActual = 0;
      }
    }    
    
    /// Devuelve el personaje a su posicin inicial
    public  void Reiniciar()
    {
      x = xOriginal;
      y = yOriginal;
    } 

    
    /// Dibuja el elemento grafico en su posicion actual en pantalla oculta
    public  void DibujarOculta()
    {
      if (activo==false) return;
      if (contieneSecuencia)
        secuencia[direccion][fotogramaActual].DibujarOculta(x, y);
      else if (contieneImagen)
        miImagen.DibujarOculta(x, y);
      else
        Hardware.ErrorFatal("Se ha intentado dibujar una imagen no cargada!");
    } 
    
    /// Dibuja el elemento grafico en una posicion cualquiera en pantalla oculta
    public  void DibujarOculta(short nuevaX, short nuevaY)
    {
      MoverA(nuevaX, nuevaY);
      DibujarOculta();
    }

    /// Comprueba si ha chocado con otro elemento grfico
    public bool ColisionCon(ElemGrafico otroElem)
    {
      // No se debe chocar con un elemento oculto      
      if ((activo == false) || (otroElem.activo == false))
        return false;
      // Ahora ya compruebo coordenadas
      if ((otroElem.x+otroElem.ancho > x)
          && (otroElem.y+otroElem.alto > y)
          && (x+ancho > otroElem.x)
          && (y+alto > otroElem.y))
        return true;
      else
        return false;
    }
    
    /// Prepara el siguiente fotograma, para animar el movimiento de
    /// un personaje
    public void SiguienteFotograma()
    {
      if (fotogramaActual < secuencia[direccion].Length -1 )
        fotogramaActual ++;
      else 
        fotogramaActual = 0;
    }
    
    public void Mover()
    {
      // Para ser redefinido en las clases "hijas"
    }
    
    /// Devuelve el valor de x
    public short GetX()
    {
      return x;
    }
    
    /// Devuelve el valor de y
    public short GetY()
    {
      return y;
    }
    
    /// Cambia el ancho y el alto
    public  void SetAnchoAlto(short an, short al)
    {
      alto = al;
      ancho = an;
    }
    
    /// Devuelve si est activo
    public  bool GetActivo()
    {
      return activo;
    }
    
    /// Cambia si est activo (visible) o no (muerto / no visible)
    public  void SetActivo(bool a)
    {
      activo = a;
    }
    
    
    /// Devuelve la direccion del personaje
    public  byte GetDireccion()
    {
      return direccion;
    }
    
    
} /* end class ElemGrafico */
