/** 
 *   Mapa: Mapa de juego
 *  
 *   @see Hardware ElemGrafico Juego
 *   @author 1-DAI IES San Vicente 2009/10
 */
 
/* --------------------------------------------------         
   Parte de Death Pit - Remake
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.14  23-Dic-2009  Nacho Cabanes
                      Primera versin del Mapa, extraida de "Juego.cs"
   0.16  01-Feb-2010  Nacho Cabanes
                      EsPosibleMover usa coordenadas inicial y final
                      Aadidas constantes como CASILLA_ESPACIO
 ---------------------------------------------------- */

public class Mapa
{

    public const byte CASILLA_ESPACIO = 0;
    public const byte CASILLA_HORIZ = 1;
    public const byte CASILLA_HORIZ2 = 2;
    public const byte CASILLA_VERT = 3;
    public const byte CASILLA_ESCALERA = 4;

    byte [,,,] mapa =
    {      // Comienzo del mapa de casillas
      {    // Primera fila de habitaciones
        {  // Habitacion (0,0)
          {1,2,1,1,1,2,1},
          {3,0,0,0,0,0,0},
          {3,4,3,4,3,1,1},
          {3,4,0,4,0,0,0},
          {3,4,3,4,3,4,3},
          {3,4,0,0,0,4,3},
          {3,1,1,4,1,4,3}
        },
        {  // Habitacion (0,1)
          {1,2,1,1,1,2,1},
          {0,0,0,0,0,0,3},
          {1,1,3,4,3,4,3},
          {0,0,3,4,3,4,3},
          {3,4,3,4,3,4,3},
          {3,4,3,4,3,4,3},
          {3,1,3,4,3,4,3}
        },
      },
      {    // Segunda fila de habitaciones
        {  // Habitacion (1,0)
          {3,2,1,4,1,4,1},
          {3,0,0,0,0,0,0},
          {3,1,1,4,3,1,1},
          {3,0,0,4,3,0,0},
          {3,4,3,4,3,4,3},
          {3,4,3,4,0,4,3},
          {1,1,1,1,1,1,3}
        },
        {  // Habitacion (1,1)
          {1,2,1,4,1,4,1},
          {0,0,0,0,0,0,3},
          {1,1,1,4,1,1,3},
          {0,0,0,4,0,0,3},
          {1,4,3,4,3,4,3},
          {3,4,3,4,0,4,3},
          {1,1,1,1,1,1,3}
        },
      },
    };
    
    ElemGrafico imagenFondoHoriz,imagenFondoHorizAcido,
      imagenFondoVert, imagenEscalera;
    
    // Tamao de cada pantalla de juego
    int filasPantalla = 7;    
    int columnasPantalla = 7;
    
    // Tamao de cada "casilla" de pantalla (mapa)
    protected const byte ANCHOCASILLA = 70, 
        ALTOCASILLA = 50;

    // Tamao, en "casillas" de cada pantalla 
    protected const byte MAXFILAS = 6;
    protected const byte MAXCOLS = 6;
  
    // Posiciones lmite de movimiento del personaje
    short minX = 0, maxX = 6 * ANCHOCASILLA,
    minY = 0, maxY = 6* ALTOCASILLA;
      
          
     // Situacin de la pantalla actual dentro del mapa de habitaciones
     int filaActual = 0;
     int columnaActual = 0;

  
  
  /// Constructor: carga las imagenes del mapa
  public Mapa()
  {
      imagenFondoHoriz = new ElemGrafico("imagenes/fondo1.png");
      imagenFondoHorizAcido = new ElemGrafico("imagenes/fondo2.png");
      imagenFondoVert = new ElemGrafico("imagenes/fondo3.png");
      imagenEscalera = new ElemGrafico("imagenes/escalera.png");
  }
  

  /// Devuelve el carcter que hay en la posicin [x,y]
  public int GetPosicion(short x, short y)
  {  
    return mapa[filaActual, columnaActual, y, x];
  }

  
  /// Dibuja el mapa en pantalla
  public void Dibujar()
  {
      int fila, colum;
      
      // Dibujo el fondo de la pantalla
      for (fila=0; fila<filasPantalla; fila++)
        for (colum=0; colum<columnasPantalla; colum++)
          switch (mapa[filaActual, columnaActual, fila,colum]) {
          
            case CASILLA_ESPACIO:  // Espacio en blanco, no dibujo nada
                break;
                
            case CASILLA_HORIZ:
                imagenFondoHoriz.MoverA( (short) (colum*ANCHOCASILLA) , 
                                        (short) (fila*ALTOCASILLA));
                imagenFondoHoriz.DibujarOculta();
                break;
                
            case CASILLA_HORIZ2:
                imagenFondoHorizAcido.MoverA( (short) (colum*ANCHOCASILLA) , 
                                             (short) (fila*ALTOCASILLA));
                imagenFondoHorizAcido.DibujarOculta();
                break;
                
            case CASILLA_VERT:
                imagenFondoVert.MoverA( (short) (colum*ANCHOCASILLA) , 
                                       (short) (fila*ALTOCASILLA));
                imagenFondoVert.DibujarOculta();
                break;
                
            case CASILLA_ESCALERA:
                imagenEscalera.MoverA( (short) (colum*ANCHOCASILLA) , 
                                      (short) (fila*ALTOCASILLA));
                imagenEscalera.DibujarOculta();
                break;
           
          }
  }

  public  bool EsPosibleMover(short x, short y, short xMax, short yMax)
  {
      short posX = (short) (x/ANCHOCASILLA);
	  short posY = (short) (y/ALTOCASILLA);
	  short posXmax = (short) ((xMax-1)/ANCHOCASILLA);
	  short posYmax = (short) ((yMax-1)/ALTOCASILLA);
      
      //Para que no se salga por los bordes de la pantalla
      if ((posX < 0) || (posY < 0) || (posX >= MAXCOLS) || (posY >= MAXFILAS))
          return false;
      //Controlar colision con otros bloques
      if ( ((GetPosicion(posX,posY) != CASILLA_ESPACIO) &&
            (GetPosicion(posX,posY) != CASILLA_ESCALERA) ) 
           ||
           ((GetPosicion(posX,posYmax) != CASILLA_ESPACIO) &&
            (GetPosicion(posX,posYmax) != CASILLA_ESCALERA) ) 
           ||
           ((GetPosicion(posXmax,posY) != CASILLA_ESPACIO) &&
            (GetPosicion(posXmax,posY) != CASILLA_ESCALERA) ) 
           ||
           ((GetPosicion(posXmax,posYmax) != CASILLA_ESPACIO) &&
            (GetPosicion(posXmax,posYmax) != CASILLA_ESCALERA) ) 
          )
          
                return false;
      
      // Si no hay obstaculos, se puede mover a esa posicion
      return true;
  }  
  
  
  
  public  void IrAHabitacion(short columnaHab, short filaHab)
  {
      filaActual = filaHab;
      columnaActual = columnaHab;
  }
  
  
  
  public  void IrAHabitacionDerecha()
  {
      if (columnaActual < MAXCOLS)
          columnaActual++;
      
  }
  
  
  public  void IrAHabitacionIzquierda()
  {
      if (columnaActual > 0)
          columnaActual--;
  }
  
  
  public  void IrAHabitacionArriba()
  {
      if (filaActual > 0)
          filaActual--;
  }
  
  
  public  void IrAHabitacionAbajo()
  {
      if (filaActual < MAXFILAS)
          filaActual++;
  }

  
  // Mxima posicin horizontal a la que se puede mover un personaje o enemigo
  public  short GetMaxX()
  {
      return maxX;
  }
  
  
  // Mxima posicin vertical la que se puede mover un personaje o enemigo
  public  short GetMaxY()
  {
      return maxY;
  }
  
} /* fin de la clase Mapa */
